@@ -5,7 +5,8 @@ from ..mixins import UserNotifyMixin
 
 
 class NotifyUserListParticipant(UserNotifyMixin):
-    user_list = models.ForeignKey(NotifyUserList, on_delete=models.CASCADE, related_name='participants', verbose_name='Список пользователей для рассылки')
+    user_list = models.ForeignKey(NotifyUserList, on_delete=models.CASCADE, related_name='participants',
+                                  verbose_name='Список пользователей для рассылки')
     created_at = models.DateTimeField(auto_now_add=True, verbose_name='Дата создания')
     user = models.ForeignKey(settings.AUTH_USER_MODEL, blank=True, null=True, on_delete=models.SET_NULL,
                              related_name='user_lists', verbose_name='Пользователь (получатель)')
@@ -16,5 +17,5 @@ class NotifyUserListParticipant(UserNotifyMixin):
         return f'Участник рассылки {self.pk}'
 
     class Meta:
-        verbose_name = 'Участник списка пользователей'
-        verbose_name_plural = 'Участники списка пользователей'
+        verbose_name = 'Дополнительный участник списка рассылки'
+        verbose_name_plural = 'Дополнительные участники списка рассылки'
